@@ -15,14 +15,14 @@
   See LICENSE for licensing information.
 
 <Purpose>
-  A central location for all format-related checking of TUF objects.
-  Note: 'formats.py' depends heavily on 'schema.py', so the 'schema.py'
-  module should be read and understood before tackling this module.
+  A central location for all format-related checking of securesystemslib
+  objects. Note: 'formats.py' depends heavily on 'schema.py', so the
+  'schema.py' module should be read and understood before tackling this module.
 
   'formats.py' can be broken down into three sections.  (1) Schemas and object
   matching.  (2) Classes that represent Role Metadata and help produce
-  correctly formatted files.  (3) Functions that help produce or verify TUF
-  objects.
+  correctly formatted files.  (3) Functions that help produce or verify
+  securesystemslib objects.
 
   The first section deals with schemas and object matching based on format.
   There are two ways of checking the format of objects.  The first method
@@ -57,7 +57,7 @@
   schema to ensure correctly formatted metadata.
 
   The last section contains miscellaneous functions related to the format of
-  TUF objects.
+  securesystemslib objects.
   Example:
 
   signable_object = make_signable(unsigned_object)
@@ -74,18 +74,20 @@ from __future__ import unicode_literals
 import binascii
 import calendar
 import re
-import string
 import datetime
 import time
 import six
 
-import securesystemslib.schema as SCHEMA
-import securesystemslib.exceptions
+from securesystemslib import exceptions
+from securesystemslib import schema as SCHEMA
 
 # Note that in the schema definitions below, the 'SCHEMA.Object' types allow
 # additional keys which are not defined. Thus, any additions to them will be
 # easily backwards compatible with clients that are already deployed.
 
+ANY_STRING_SCHEMA = SCHEMA.AnyString()
+LIST_OF_ANY_STRING_SCHEMA = SCHEMA.ListOf(ANY_STRING_SCHEMA)
+
 # A datetime in 'YYYY-MM-DDTHH:MM:SSZ' ISO 8601 format.  The "Z" zone designator
 # for the zero UTC offset is always used (i.e., a numerical offset is not
 # supported.)  Example: '2015-10-21T13:20:00Z'.  Note:  This is a simple format
@@ -99,15 +101,20 @@ ISO8601_DATETIME_SCHEMA = SCHEMA.RegularExpression(r'\d{4}-\d{2}-\d{2}T\d{2}:\d{
 UNIX_TIMESTAMP_SCHEMA = SCHEMA.Integer(lo=0, hi=2147483647)
 
 # A hexadecimal value in '23432df87ab..' format.
-HASH_SCHEMA = SCHEMA.RegularExpression(r'[a-fA-F0-9]+')
+HEX_SCHEMA = SCHEMA.RegularExpression(r'[a-fA-F0-9]+')
+
+HASH_SCHEMA = HEX_SCHEMA
 
 # A dict in {'sha256': '23432df87ab..', 'sha512': '34324abc34df..', ...} format.
 HASHDICT_SCHEMA = SCHEMA.DictOf(
   key_schema = SCHEMA.AnyString(),
   value_schema = HASH_SCHEMA)
 
-# A hexadecimal value in '23432df87ab..' format.
-HEX_SCHEMA = SCHEMA.RegularExpression(r'[a-fA-F0-9]+')
+# Uniform Resource Locator identifier (e.g., 'https://www.updateframework.com/').
+# TODO: Some level of restriction here would be good....  Note that I pulled
+#       this from securesystemslib, since it's neither sophisticated nor used
+#       by anyone else.
+URL_SCHEMA = SCHEMA.AnyString()
 
 # A key identifier (e.g., a hexadecimal value identifying an RSA key).
 KEYID_SCHEMA = HASH_SCHEMA
@@ -119,31 +126,10 @@ KEYIDS_SCHEMA = SCHEMA.ListOf(KEYID_SCHEMA)
 # 'rsassa-pss-sha256' is one of the signing schemes for key type 'rsa').
 SCHEME_SCHEMA = SCHEMA.AnyString()
 
-# A relative file path (e.g., 'metadata/root/').
-RELPATH_SCHEMA = SCHEMA.AnyString()
-RELPATHS_SCHEMA = SCHEMA.ListOf(RELPATH_SCHEMA)
-
-# An absolute path.
-PATH_SCHEMA = SCHEMA.AnyString()
+# A path string, whether relative or absolute, e.g. 'metadata/root/'
+PATH_SCHEMA = SCHEMA.AnyNonemptyString()
 PATHS_SCHEMA = SCHEMA.ListOf(PATH_SCHEMA)
 
-# Uniform Resource Locator identifier (e.g., 'https://www.updateframework.com/').
-URL_SCHEMA = SCHEMA.AnyString()
-
-# A dictionary holding version information.
-VERSION_SCHEMA = SCHEMA.Object(
-  object_name = 'VERSION_SCHEMA',
-  major = SCHEMA.Integer(lo=0),
-  minor = SCHEMA.Integer(lo=0),
-  fix = SCHEMA.Integer(lo=0))
-
-# An integer representing the numbered version of a metadata file.
-# Must be 1, or greater.
-METADATAVERSION_SCHEMA = SCHEMA.Integer(lo=0)
-
-# An integer representing length.  Must be 0, or greater.
-LENGTH_SCHEMA = SCHEMA.Integer(lo=0)
-
 # An integer representing logger levels, such as logging.CRITICAL (=50).
 # Must be between 0 and 50.
 LOGLEVEL_SCHEMA = SCHEMA.Integer(lo=0, hi=50)
@@ -162,37 +148,24 @@ TEXT_SCHEMA = SCHEMA.AnyString()
 HASHALGORITHMS_SCHEMA = SCHEMA.ListOf(SCHEMA.OneOf(
   [SCHEMA.String('md5'), SCHEMA.String('sha1'),
    SCHEMA.String('sha224'), SCHEMA.String('sha256'),
-   SCHEMA.String('sha384'), SCHEMA.String('sha512')]))
+   SCHEMA.String('sha384'), SCHEMA.String('sha512'),
+   SCHEMA.String('blake2s'), SCHEMA.String('blake2b'),
+   SCHEMA.String('blake2b-256')]))
 
-# The contents of an encrypted TUF key.  Encrypted TUF keys are saved to files
+# The contents of an encrypted key.  Encrypted keys are saved to files
 # in this format.
 ENCRYPTEDKEY_SCHEMA = SCHEMA.AnyString()
 
 # A value that is either True or False, on or off, etc.
 BOOLEAN_SCHEMA = SCHEMA.Boolean()
 
-# A role's threshold value (i.e., the minimum number
-# of signatures required to sign a metadata file).
-# Must be 1 and greater.
-THRESHOLD_SCHEMA = SCHEMA.Integer(lo=1)
-
-# A string representing a role's name.
-ROLENAME_SCHEMA = SCHEMA.AnyString()
-
 # The minimum number of bits for an RSA key.  Must be 2048 bits, or greater
 # (recommended by TUF).  Recommended RSA key sizes:
 # http://www.emc.com/emc-plus/rsa-labs/historical/twirl-and-rsa-key-size.htm#table1
 RSAKEYBITS_SCHEMA = SCHEMA.Integer(lo=2048)
 
-# The supported ECDSA signature schemes (ecdsa-sha2-nistp256 is supported by
-# default).
-ECDSA_SCHEME_SCHEMA = SCHEMA.OneOf([SCHEMA.String('ecdsa-sha2-nistp256')])
-
-# The number of hashed bins, or the number of delegated roles.  See
-# delegate_hashed_bins() in 'repository_tool.py' for an example.  Note:
-# Tools may require further restrictions on the number of bins, such
-# as requiring them to be a power of 2.
-NUMBINS_SCHEMA = SCHEMA.Integer(lo=1)
+# The supported ECDSA signature schemes
+ECDSA_SCHEME_SCHEMA = SCHEMA.RegularExpression(r'ecdsa-sha2-nistp(256|384)')
 
 # A pyca-cryptography signature.
 PYCACRYPTOSIGNATURE_SCHEMA = SCHEMA.AnyBytes()
@@ -224,13 +197,13 @@ PUBLIC_KEYVAL_SCHEMA = SCHEMA.Object(
   public = SCHEMA.AnyString(),
   private = SCHEMA.Optional(SCHEMA.String("")))
 
-# Supported TUF key types.
+# Supported securesystemslib key types.
 KEYTYPE_SCHEMA = SCHEMA.OneOf(
-  [SCHEMA.String('rsa'), SCHEMA.String('ed25519'),
-   SCHEMA.String('ecdsa-sha2-nistp256')])
+  [SCHEMA.String('rsa'), SCHEMA.String('ed25519'), SCHEMA.String('ecdsa'),
+   SCHEMA.RegularExpression(r'ecdsa-sha2-nistp(256|384)')])
 
-# A generic TUF key.  All TUF keys should be saved to metadata files in this
-# format.
+# A generic securesystemslib key.  All securesystemslib keys should be saved to
+# metadata files in this format.
 KEY_SCHEMA = SCHEMA.Object(
   object_name = 'KEY_SCHEMA',
   keytype = SCHEMA.AnyString(),
@@ -248,8 +221,9 @@ PUBLIC_KEY_SCHEMA = SCHEMA.Object(
   keyval = PUBLIC_KEYVAL_SCHEMA,
   expires = SCHEMA.Optional(ISO8601_DATETIME_SCHEMA))
 
-# A TUF key object.  This schema simplifies validation of keys that may be one
-# of the supported key types.  Supported key types: 'rsa', 'ed25519'.
+# A securesystemslib key object.  This schema simplifies validation of keys
+# that may be one of the supported key types.  Supported key types: 'rsa',
+# 'ed25519'.
 ANYKEY_SCHEMA = SCHEMA.Object(
   object_name = 'ANYKEY_SCHEMA',
   keytype = KEYTYPE_SCHEMA,
@@ -259,13 +233,15 @@ ANYKEY_SCHEMA = SCHEMA.Object(
   keyval = KEYVAL_SCHEMA,
   expires = SCHEMA.Optional(ISO8601_DATETIME_SCHEMA))
 
-# A list of TUF key objects.
+# A list of securesystemslib key objects.
 ANYKEYLIST_SCHEMA = SCHEMA.ListOf(ANYKEY_SCHEMA)
 
 # RSA signature schemes.
-RSA_SCHEME_SCHEMA = SCHEMA.OneOf([SCHEMA.String('rsassa-pss-sha256')])
+RSA_SCHEME_SCHEMA = SCHEMA.OneOf([
+  SCHEMA.RegularExpression(r'rsassa-pss-(md5|sha1|sha224|sha256|sha384|sha512)'),
+  SCHEMA.RegularExpression(r'rsa-pkcs1v15-(md5|sha1|sha224|sha256|sha384|sha512)')])
 
-# An RSA TUF key.
+# An RSA securesystemslib key.
 RSAKEY_SCHEMA = SCHEMA.Object(
   object_name = 'RSAKEY_SCHEMA',
   keytype = SCHEMA.String('rsa'),
@@ -274,10 +250,11 @@ RSAKEY_SCHEMA = SCHEMA.Object(
   keyid_hash_algorithms = SCHEMA.Optional(HASHALGORITHMS_SCHEMA),
   keyval = KEYVAL_SCHEMA)
 
-# An ECDSA TUF key.
+# An ECDSA securesystemslib key.
 ECDSAKEY_SCHEMA = SCHEMA.Object(
   object_name = 'ECDSAKEY_SCHEMA',
-  keytype = SCHEMA.String('ecdsa-sha2-nistp256'),
+  keytype = SCHEMA.OneOf([SCHEMA.String('ecdsa'),
+                          SCHEMA.RegularExpression(r'ecdsa-sha2-nistp(256|384)')]),
   scheme = ECDSA_SCHEME_SCHEMA,
   keyid = KEYID_SCHEMA,
   keyid_hash_algorithms = SCHEMA.Optional(HASHALGORITHMS_SCHEMA),
@@ -295,17 +272,11 @@ ED25519SIGNATURE_SCHEMA = SCHEMA.LengthBytes(64)
 # An ECDSA signature.
 ECDSASIGNATURE_SCHEMA = SCHEMA.AnyBytes()
 
-# Required installation libraries expected by the repository tools and other
-# cryptography modules.
-REQUIRED_LIBRARIES_SCHEMA = SCHEMA.ListOf(SCHEMA.OneOf(
-  [SCHEMA.String('general'), SCHEMA.String('ed25519'), SCHEMA.String('rsa'),
-   SCHEMA.String('ecdsa-sha2-nistp256')]))
-
 # Ed25519 signature schemes.  The vanilla Ed25519 signature scheme is currently
 # supported.
 ED25519_SIG_SCHEMA = SCHEMA.OneOf([SCHEMA.String('ed25519')])
 
-# An ed25519 TUF key.
+# An ed25519 key.
 ED25519KEY_SCHEMA = SCHEMA.Object(
   object_name = 'ED25519KEY_SCHEMA',
   keytype = SCHEMA.String('ed25519'),
@@ -314,39 +285,114 @@ ED25519KEY_SCHEMA = SCHEMA.Object(
   keyid_hash_algorithms = SCHEMA.Optional(HASHALGORITHMS_SCHEMA),
   keyval = KEYVAL_SCHEMA)
 
-# Information about target files, like file length and file hash(es).  This
-# schema allows the storage of multiple hashes for the same file (e.g., sha256
-# and sha512 may be computed for the same file and stored).
-FILEINFO_SCHEMA = SCHEMA.Object(
-  object_name = 'FILEINFO_SCHEMA',
-  length = LENGTH_SCHEMA,
-  hashes = HASHDICT_SCHEMA,
-  version = SCHEMA.Optional(METADATAVERSION_SCHEMA),
-  custom = SCHEMA.Optional(SCHEMA.Object()))
+# GPG key scheme definitions
+GPG_HASH_ALGORITHM_STRING = "pgp+SHA2"
+GPG_RSA_PUBKEY_METHOD_STRING = "pgp+rsa-pkcsv1.5"
+GPG_DSA_PUBKEY_METHOD_STRING = "pgp+dsa-fips-180-2"
+GPG_ED25519_PUBKEY_METHOD_STRING = "pgp+eddsa-ed25519"
 
-# Version information specified in "snapshot.json" for each role available on
-# the TUF repository.  The 'FILEINFO_SCHEMA' object was previously listed in
-# the snapshot role, but was switched to this object format to reduce the
-# amount of metadata that needs to be downloaded.  Listing version numbers in
-# "snapshot.json" also prevents rollback attacks for roles that clients have
-# not downloaded.
-VERSIONINFO_SCHEMA = SCHEMA.Object(
-  object_name = 'VERSIONINFO_SCHEMA',
-  version = METADATAVERSION_SCHEMA)
+def _create_gpg_pubkey_with_subkey_schema(pubkey_schema):
+  """Helper method to extend the passed public key schema with an optional
+  dictionary of sub public keys "subkeys" with the same schema."""
+  schema = pubkey_schema
+  subkey_schema_tuple =  ("subkeys", SCHEMA.Optional(
+        SCHEMA.DictOf(
+          key_schema=KEYID_SCHEMA,
+          value_schema=pubkey_schema
+          )
+        )
+      )
+  # Any subclass of `securesystemslib.schema.Object` stores the schemas that
+  # define the attributes of the object in its `_required` property, even if
+  # such a schema is of type `Optional`.
+  # TODO: Find a way that does not require to access a protected member
+  schema._required.append(subkey_schema_tuple) # pylint: disable=protected-access
+  return schema
 
-# A dict holding the version information for a particular metadata role.  The
-# dict keys hold the relative file paths, and the dict values the corresponding
-# version numbers.
-VERSIONDICT_SCHEMA = SCHEMA.DictOf(
-  key_schema = RELPATH_SCHEMA,
-  value_schema = VERSIONINFO_SCHEMA)
+GPG_RSA_PUBKEYVAL_SCHEMA = SCHEMA.Object(
+  object_name = "GPG_RSA_PUBKEYVAL_SCHEMA",
+  e = SCHEMA.AnyString(),
+  n = HEX_SCHEMA
+)
 
-# A dict holding the information for a particular target / file.  The dict keys
-# hold the relative file paths, and the dict values the corresponding file
-# information.
-FILEDICT_SCHEMA = SCHEMA.DictOf(
-  key_schema = RELPATH_SCHEMA,
-  value_schema = FILEINFO_SCHEMA)
+# We have to define GPG_RSA_PUBKEY_SCHEMA in two steps, because it is
+# self-referential. Here we define a shallow _GPG_RSA_PUBKEY_SCHEMA, which we
+# use below to create the self-referential GPG_RSA_PUBKEY_SCHEMA.
+_GPG_RSA_PUBKEY_SCHEMA = SCHEMA.Object(
+  object_name = "GPG_RSA_PUBKEY_SCHEMA",
+  type = SCHEMA.String("rsa"),
+  method = SCHEMA.String(GPG_RSA_PUBKEY_METHOD_STRING),
+  hashes = SCHEMA.ListOf(SCHEMA.String(GPG_HASH_ALGORITHM_STRING)),
+  creation_time = SCHEMA.Optional(UNIX_TIMESTAMP_SCHEMA),
+  validity_period = SCHEMA.Optional(SCHEMA.Integer(lo=0)),
+  keyid = KEYID_SCHEMA,
+  keyval = SCHEMA.Object(
+      public = GPG_RSA_PUBKEYVAL_SCHEMA,
+      private = SCHEMA.String("")
+    )
+)
+GPG_RSA_PUBKEY_SCHEMA = _create_gpg_pubkey_with_subkey_schema(
+    _GPG_RSA_PUBKEY_SCHEMA)
+
+GPG_DSA_PUBKEYVAL_SCHEMA = SCHEMA.Object(
+  object_name = "GPG_DSA_PUBKEYVAL_SCHEMA",
+  y = HEX_SCHEMA,
+  p = HEX_SCHEMA,
+  q = HEX_SCHEMA,
+  g = HEX_SCHEMA
+)
+
+# C.f. comment above _GPG_RSA_PUBKEY_SCHEMA definition
+_GPG_DSA_PUBKEY_SCHEMA = SCHEMA.Object(
+  object_name = "GPG_DSA_PUBKEY_SCHEMA",
+  type = SCHEMA.String("dsa"),
+  method = SCHEMA.String(GPG_DSA_PUBKEY_METHOD_STRING),
+  hashes = SCHEMA.ListOf(SCHEMA.String(GPG_HASH_ALGORITHM_STRING)),
+  creation_time = SCHEMA.Optional(UNIX_TIMESTAMP_SCHEMA),
+  validity_period = SCHEMA.Optional(SCHEMA.Integer(lo=0)),
+  keyid = KEYID_SCHEMA,
+  keyval = SCHEMA.Object(
+      public = GPG_DSA_PUBKEYVAL_SCHEMA,
+      private = SCHEMA.String("")
+    )
+)
+
+GPG_DSA_PUBKEY_SCHEMA = _create_gpg_pubkey_with_subkey_schema(
+    _GPG_DSA_PUBKEY_SCHEMA)
+
+GPG_ED25519_PUBKEYVAL_SCHEMA = SCHEMA.Object(
+  object_name = "GPG_ED25519_PUBKEYVAL_SCHEMA",
+  q = HEX_SCHEMA,
+)
+
+# C.f. comment above _GPG_RSA_PUBKEY_SCHEMA definition
+_GPG_ED25519_PUBKEY_SCHEMA = SCHEMA.Object(
+  object_name = "GPG_ED25519_PUBKEY_SCHEMA",
+  type = SCHEMA.String("eddsa"),
+  method = SCHEMA.String(GPG_ED25519_PUBKEY_METHOD_STRING),
+  hashes = SCHEMA.ListOf(SCHEMA.String(GPG_HASH_ALGORITHM_STRING)),
+  creation_time = SCHEMA.Optional(UNIX_TIMESTAMP_SCHEMA),
+  validity_period = SCHEMA.Optional(SCHEMA.Integer(lo=0)),
+  keyid = KEYID_SCHEMA,
+  keyval = SCHEMA.Object(
+      public = GPG_ED25519_PUBKEYVAL_SCHEMA,
+      private = SCHEMA.String("")
+    )
+)
+GPG_ED25519_PUBKEY_SCHEMA = _create_gpg_pubkey_with_subkey_schema(
+    _GPG_ED25519_PUBKEY_SCHEMA)
+
+GPG_PUBKEY_SCHEMA = SCHEMA.OneOf([GPG_RSA_PUBKEY_SCHEMA,
+    GPG_DSA_PUBKEY_SCHEMA, GPG_ED25519_PUBKEY_SCHEMA])
+
+GPG_SIGNATURE_SCHEMA = SCHEMA.Object(
+    object_name = "SIGNATURE_SCHEMA",
+    keyid = KEYID_SCHEMA,
+    short_keyid = SCHEMA.Optional(KEYID_SCHEMA),
+    other_headers = HEX_SCHEMA,
+    signature = HEX_SCHEMA,
+    info = SCHEMA.Optional(SCHEMA.Any()),
+  )
 
 # A single signature of an object.  Indicates the signature, and the KEYID of
 # the signing key.  I debated making the signature schema not contain the key
@@ -359,174 +405,43 @@ SIGNATURE_SCHEMA = SCHEMA.Object(
   keyid = KEYID_SCHEMA,
   sig = HEX_SCHEMA)
 
-# List of SIGNATURE_SCHEMA.
-SIGNATURES_SCHEMA = SCHEMA.ListOf(SIGNATURE_SCHEMA)
-
-# A schema holding the result of checking the signatures of a particular
-# 'SIGNABLE_SCHEMA' role.
-# For example, how many of the signatures for the 'Target' role are
-# valid?  This SCHEMA holds this information.  See 'sig.py' for
-# more information.
-SIGNATURESTATUS_SCHEMA = SCHEMA.Object(
-  object_name = 'SIGNATURESTATUS_SCHEMA',
-  threshold = SCHEMA.Integer(),
-  good_sigs = KEYIDS_SCHEMA,
-  bad_sigs = KEYIDS_SCHEMA,
-  unknown_sigs = KEYIDS_SCHEMA,
-  untrusted_sigs = KEYIDS_SCHEMA)
-
-# A signable object.  Holds the signing role and its associated signatures.
-SIGNABLE_SCHEMA = SCHEMA.Object(
-  object_name = 'SIGNABLE_SCHEMA',
-  signed = SCHEMA.Any(),
-  signatures = SCHEMA.ListOf(SIGNATURE_SCHEMA))
-
 # A dict where the dict keys hold a keyid and the dict values a key object.
 KEYDICT_SCHEMA = SCHEMA.DictOf(
   key_schema = KEYID_SCHEMA,
   value_schema = KEY_SCHEMA)
 
-# The format used by the key database to store keys.  The dict keys hold a key
-# identifier and the dict values any object.  The key database should store
-# key objects in the values (e.g., 'RSAKEY_SCHEMA', 'DSAKEY_SCHEMA').
-KEYDB_SCHEMA = SCHEMA.DictOf(
+ANY_SIGNATURE_SCHEMA = SCHEMA.OneOf([SIGNATURE_SCHEMA,
+    GPG_SIGNATURE_SCHEMA])
+
+# List of ANY_SIGNATURE_SCHEMA.
+SIGNATURES_SCHEMA = SCHEMA.ListOf(ANY_SIGNATURE_SCHEMA)
+
+# A signable object.  Holds the signing role and its associated signatures.
+SIGNABLE_SCHEMA = SCHEMA.Object(
+  object_name = 'SIGNABLE_SCHEMA',
+  signed = SCHEMA.Any(),
+  signatures = SIGNATURES_SCHEMA)
+
+# Note: Verification keys can have private portions but in case of GPG we
+# only have a PUBKEY_SCHEMA (because we never export private gpg keys from
+# the gpg keyring)
+ANY_VERIFICATION_KEY_SCHEMA = SCHEMA.OneOf([ANYKEY_SCHEMA,
+    GPG_PUBKEY_SCHEMA])
+
+VERIFICATION_KEY_DICT_SCHEMA = SCHEMA.DictOf(
   key_schema = KEYID_SCHEMA,
-  value_schema = SCHEMA.Any())
+  value_schema = ANY_VERIFICATION_KEY_SCHEMA)
 
-# A path hash prefix is a hexadecimal string.
-PATH_HASH_PREFIX_SCHEMA = HEX_SCHEMA
+ANY_KEYDICT_SCHEMA = SCHEMA.OneOf([KEYDICT_SCHEMA,
+    VERIFICATION_KEY_DICT_SCHEMA])
 
-# A list of path hash prefixes.
-PATH_HASH_PREFIXES_SCHEMA = SCHEMA.ListOf(PATH_HASH_PREFIX_SCHEMA)
+ANY_PUBKEY_SCHEMA = SCHEMA.OneOf([PUBLIC_KEY_SCHEMA, GPG_PUBKEY_SCHEMA])
 
-# Role object in {'keyids': [keydids..], 'name': 'ABC', 'threshold': 1,
-# 'paths':[filepaths..]} format.
-ROLE_SCHEMA = SCHEMA.Object(
-  object_name = 'ROLE_SCHEMA',
-  name = SCHEMA.Optional(ROLENAME_SCHEMA),
-  keyids = KEYIDS_SCHEMA,
-  threshold = THRESHOLD_SCHEMA,
-  backtrack = SCHEMA.Optional(BOOLEAN_SCHEMA),
-  paths = SCHEMA.Optional(RELPATHS_SCHEMA),
-  path_hash_prefixes = SCHEMA.Optional(PATH_HASH_PREFIXES_SCHEMA))
+ANY_PUBKEY_DICT_SCHEMA = SCHEMA.DictOf(
+  key_schema = KEYID_SCHEMA,
+  value_schema = ANY_PUBKEY_SCHEMA)
 
-# A dict of roles where the dict keys are role names and the dict values holding
-# the role data/information.
-ROLEDICT_SCHEMA = SCHEMA.DictOf(
-  key_schema = ROLENAME_SCHEMA,
-  value_schema = ROLE_SCHEMA)
 
-# Like ROLEDICT_SCHEMA, except that ROLE_SCHEMA instances are stored in order.
-ROLELIST_SCHEMA = SCHEMA.ListOf(ROLE_SCHEMA)
-
-# The delegated roles of a Targets role (a parent).
-DELEGATIONS_SCHEMA = SCHEMA.Object(
-  keys = KEYDICT_SCHEMA,
-  roles = ROLELIST_SCHEMA)
-
-# The fileinfo format of targets specified in the repository and
-# developer tools.  The second element of this list holds custom data about the
-# target, such as file permissions, author(s), last modified, etc.
-CUSTOM_SCHEMA = SCHEMA.Object()
-
-PATH_FILEINFO_SCHEMA = SCHEMA.DictOf(
-  key_schema = RELPATH_SCHEMA,
-  value_schema = CUSTOM_SCHEMA)
-
-# TUF roledb
-ROLEDB_SCHEMA = SCHEMA.Object(
-  object_name = 'ROLEDB_SCHEMA',
-  keyids = SCHEMA.Optional(KEYIDS_SCHEMA),
-  signing_keyids = SCHEMA.Optional(KEYIDS_SCHEMA),
-  previous_keyids = SCHEMA.Optional(KEYIDS_SCHEMA),
-  threshold = SCHEMA.Optional(THRESHOLD_SCHEMA),
-  previous_threshold = SCHEMA.Optional(THRESHOLD_SCHEMA),
-  version = SCHEMA.Optional(METADATAVERSION_SCHEMA),
-  expires = SCHEMA.Optional(ISO8601_DATETIME_SCHEMA),
-  signatures = SCHEMA.Optional(SIGNATURES_SCHEMA),
-  paths = SCHEMA.Optional(SCHEMA.OneOf([RELPATHS_SCHEMA, PATH_FILEINFO_SCHEMA])),
-  path_hash_prefixes = SCHEMA.Optional(PATH_HASH_PREFIXES_SCHEMA),
-  delegations = SCHEMA.Optional(DELEGATIONS_SCHEMA),
-  partial_loaded = SCHEMA.Optional(BOOLEAN_SCHEMA))
-
-# Root role: indicates root keys and top-level roles.
-ROOT_SCHEMA = SCHEMA.Object(
-  object_name = 'ROOT_SCHEMA',
-  _type = SCHEMA.String('root'),
-  version = METADATAVERSION_SCHEMA,
-  consistent_snapshot = BOOLEAN_SCHEMA,
-  expires = ISO8601_DATETIME_SCHEMA,
-  keys = KEYDICT_SCHEMA,
-  roles = ROLEDICT_SCHEMA)
-
-# Targets role: Indicates targets and delegates target paths to other roles.
-TARGETS_SCHEMA = SCHEMA.Object(
-  object_name = 'TARGETS_SCHEMA',
-  _type = SCHEMA.String('targets'),
-  version = METADATAVERSION_SCHEMA,
-  expires = ISO8601_DATETIME_SCHEMA,
-  targets = FILEDICT_SCHEMA,
-  delegations = SCHEMA.Optional(DELEGATIONS_SCHEMA))
-
-# Snapshot role: indicates the latest versions of all metadata (except
-# timestamp).
-SNAPSHOT_SCHEMA = SCHEMA.Object(
-  object_name = 'SNAPSHOT_SCHEMA',
-  _type = SCHEMA.String('snapshot'),
-  version = METADATAVERSION_SCHEMA,
-  expires = ISO8601_DATETIME_SCHEMA,
-  meta = VERSIONDICT_SCHEMA)
-
-# Timestamp role: indicates the latest version of the snapshot file.
-TIMESTAMP_SCHEMA = SCHEMA.Object(
-  object_name = 'TIMESTAMP_SCHEMA',
-  _type = SCHEMA.String('timestamp'),
-  version = METADATAVERSION_SCHEMA,
-  expires = ISO8601_DATETIME_SCHEMA,
-  meta = FILEDICT_SCHEMA)
-
-# project.cfg file: stores information about the project in a json dictionary
-PROJECT_CFG_SCHEMA = SCHEMA.Object(
-    object_name = 'PROJECT_CFG_SCHEMA',
-    project_name = SCHEMA.AnyString(),
-    layout_type = SCHEMA.OneOf([SCHEMA.String('repo-like'), SCHEMA.String('flat')]),
-    targets_location = PATH_SCHEMA,
-    metadata_location = PATH_SCHEMA,
-    prefix = PATH_SCHEMA,
-    public_keys = KEYDICT_SCHEMA,
-    threshold = SCHEMA.Integer(lo = 0, hi = 2)
-    )
-
-# A schema containing information a repository mirror may require,
-# such as a url, the path of the directory metadata files, etc.
-MIRROR_SCHEMA = SCHEMA.Object(
-  object_name = 'MIRROR_SCHEMA',
-  url_prefix = URL_SCHEMA,
-  metadata_path = RELPATH_SCHEMA,
-  targets_path = RELPATH_SCHEMA,
-  confined_target_dirs = RELPATHS_SCHEMA,
-  custom = SCHEMA.Optional(SCHEMA.Object()))
-
-# A dictionary of mirrors where the dict keys hold the mirror's name and
-# and the dict values the mirror's data (i.e., 'MIRROR_SCHEMA').
-# The repository class of 'updater.py' accepts dictionaries
-# of this type provided by the TUF client.
-MIRRORDICT_SCHEMA = SCHEMA.DictOf(
-  key_schema = SCHEMA.AnyString(),
-  value_schema = MIRROR_SCHEMA)
-
-# A Mirrorlist: indicates all the live mirrors, and what documents they
-# serve.
-MIRRORLIST_SCHEMA = SCHEMA.Object(
-  object_name = 'MIRRORLIST_SCHEMA',
-  _type = SCHEMA.String('mirrors'),
-  version = METADATAVERSION_SCHEMA,
-  expires = ISO8601_DATETIME_SCHEMA,
-  mirrors = SCHEMA.ListOf(MIRROR_SCHEMA))
-
-# Any of the role schemas (e.g., TIMESTAMP_SCHEMA, SNAPSHOT_SCHEMA, etc.)
-ANYROLE_SCHEMA = SCHEMA.OneOf([ROOT_SCHEMA, TARGETS_SCHEMA, SNAPSHOT_SCHEMA,
-                               TIMESTAMP_SCHEMA, MIRROR_SCHEMA])
 
 
 
@@ -561,7 +476,7 @@ def datetime_to_unix_timestamp(datetime_object):
   # Raise 'securesystemslib.exceptions.FormatError' if not.
   if not isinstance(datetime_object, datetime.datetime):
     message = repr(datetime_object) + ' is not a datetime.datetime() object.'
-    raise securesystemslib.exceptions.FormatError(message)
+    raise exceptions.FormatError(message)
 
   unix_timestamp = calendar.timegm(datetime_object.timetuple())
 
@@ -599,7 +514,7 @@ def unix_timestamp_to_datetime(unix_timestamp):
 
   # Is 'unix_timestamp' properly formatted?
   # Raise 'securesystemslib.exceptions.FormatError' if there is a mismatch.
-  securesystemslib.formats.UNIX_TIMESTAMP_SCHEMA.check_match(unix_timestamp)
+  UNIX_TIMESTAMP_SCHEMA.check_match(unix_timestamp)
 
   # Convert 'unix_timestamp' to a 'time.struct_time',  in UTC.  The Daylight
   # Savings Time (DST) flag is set to zero.  datetime.fromtimestamp() is not
@@ -639,7 +554,7 @@ def format_base64(data):
     return binascii.b2a_base64(data).decode('utf-8').rstrip('=\n ')
 
   except (TypeError, binascii.Error) as e:
-    raise securesystemslib.exceptions.FormatError('Invalid base64'
+    raise exceptions.FormatError('Invalid base64'
       ' encoding: ' + str(e))
 
 
@@ -668,7 +583,7 @@ def parse_base64(base64_string):
 
   if not isinstance(base64_string, six.string_types):
     message = 'Invalid argument: '+repr(base64_string)
-    raise securesystemslib.exceptions.FormatError(message)
+    raise exceptions.FormatError(message)
 
   extra = len(base64_string) % 4
   if extra:
@@ -679,7 +594,7 @@ def parse_base64(base64_string):
     return binascii.a2b_base64(base64_string.encode('utf-8'))
 
   except (TypeError, binascii.Error) as e:
-    raise securesystemslib.exceptions.FormatError('Invalid base64'
+    raise exceptions.FormatError('Invalid base64'
       ' encoding: ' + str(e))
 
 
@@ -746,7 +661,7 @@ def _encode_canonical(object, output_function):
       _encode_canonical(value, output_function)
     output_function("}")
   else:
-    raise securesystemslib.exceptions.FormatError('I cannot encode '+repr(object))
+    raise exceptions.FormatError('I cannot encode '+repr(object))
 
 
 def encode_canonical(object, output_function=None):
@@ -761,13 +676,13 @@ def encode_canonical(object, output_function=None):
     or joined into a string and returned.
 
     Note: This function should be called prior to computing the hash or
-    signature of a JSON object in TUF.  For example, generating a signature
-    of a signing role object such as 'ROOT_SCHEMA' is required to ensure
-    repeatable hashes are generated across different json module versions
-    and platforms.  Code elsewhere is free to dump JSON objects in any format
-    they wish (e.g., utilizing indentation and single quotes around object
-    keys).  These objects are only required to be in "canonical JSON" format
-    when their hashes or signatures are needed.
+    signature of a JSON object in securesystemslib.  For example, generating a
+    signature of a signing role object such as 'ROOT_SCHEMA' is required to
+    ensure repeatable hashes are generated across different json module
+    versions and platforms.  Code elsewhere is free to dump JSON objects in any
+    format they wish (e.g., utilizing indentation and single quotes around
+    object keys).  These objects are only required to be in "canonical JSON"
+    format when their hashes or signatures are needed.
 
     >>> encode_canonical("")
     '""'
@@ -809,9 +724,9 @@ def encode_canonical(object, output_function=None):
   try:
     _encode_canonical(object, output_function)
 
-  except (TypeError, securesystemslib.exceptions.FormatError) as e:
+  except (TypeError, exceptions.FormatError) as e:
     message = 'Could not encode ' + repr(object) + ': ' + str(e)
-    raise securesystemslib.exceptions.FormatError(message)
+    raise exceptions.FormatError(message)
 
   # Return the encoded 'object' as a string.
   # Note: Implies 'output_function' is None,
