@@ -35,11 +35,12 @@ import logging
 import tempfile
 import json
 
-import securesystemslib.formats
-import securesystemslib.settings
-import securesystemslib.storage
-import securesystemslib.util
-import securesystemslib.keys
+from securesystemslib import exceptions
+from securesystemslib import formats
+from securesystemslib import keys
+from securesystemslib import settings
+from securesystemslib import util
+from securesystemslib.storage import FilesystemBackend
 
 from securesystemslib import KEY_TYPE_RSA, KEY_TYPE_ED25519, KEY_TYPE_ECDSA
 
@@ -76,8 +77,8 @@ def get_password(prompt='Password: ', confirm=False):
     The password entered on the prompt.
 
   """
-  securesystemslib.formats.TEXT_SCHEMA.check_match(prompt)
-  securesystemslib.formats.BOOLEAN_SCHEMA.check_match(confirm)
+  formats.TEXT_SCHEMA.check_match(prompt)
+  formats.BOOLEAN_SCHEMA.check_match(confirm)
 
   while True:
     # getpass() prompts the user for a password without echoing
@@ -110,7 +111,7 @@ def _get_key_file_encryption_password(password, prompt, path):
                                  indicate desire to not encrypt by entering no
                                  password)
   """
-  securesystemslib.formats.BOOLEAN_SCHEMA.check_match(prompt)
+  formats.BOOLEAN_SCHEMA.check_match(prompt)
 
   # We don't want to decide which takes precedence so we fail
   if password is not None and prompt:
@@ -128,7 +129,7 @@ def _get_key_file_encryption_password(password, prompt, path):
       return None
 
   if password is not None:
-    securesystemslib.formats.PASSWORD_SCHEMA.check_match(password)
+    formats.PASSWORD_SCHEMA.check_match(password)
 
     # Fail on empty passed password. A caller should pass None to indicate the
     # desire to not encrypt.
@@ -153,7 +154,7 @@ def _get_key_file_decryption_password(password, prompt, path):
                             desire to not decrypt by entering no password)
 
   """
-  securesystemslib.formats.BOOLEAN_SCHEMA.check_match(prompt)
+  formats.BOOLEAN_SCHEMA.check_match(prompt)
 
   # We don't want to decide which takes precedence so we fail
   if password is not None and prompt:
@@ -171,7 +172,7 @@ def _get_key_file_decryption_password(password, prompt, path):
       return None
 
   if password is not None:
-    securesystemslib.formats.PASSWORD_SCHEMA.check_match(password)
+    formats.PASSWORD_SCHEMA.check_match(password)
     # No additional vetting needed. Decryption will show if it was correct.
 
   return password
@@ -216,10 +217,10 @@ def _generate_and_write_rsa_keypair(filepath=None, bits=DEFAULT_RSA_KEY_BITS,
     The private key filepath.
 
   """
-  securesystemslib.formats.RSAKEYBITS_SCHEMA.check_match(bits)
+  formats.RSAKEYBITS_SCHEMA.check_match(bits)
 
   # Generate private RSA key and extract public and private both in PEM
-  rsa_key = securesystemslib.keys.generate_rsa_key(bits)
+  rsa_key = keys.generate_rsa_key(bits)
   public = rsa_key['keyval']['public']
   private = rsa_key['keyval']['private']
 
@@ -227,26 +228,26 @@ def _generate_and_write_rsa_keypair(filepath=None, bits=DEFAULT_RSA_KEY_BITS,
   if not filepath:
     filepath = os.path.join(os.getcwd(), rsa_key['keyid'])
 
-  securesystemslib.formats.PATH_SCHEMA.check_match(filepath)
+  formats.PATH_SCHEMA.check_match(filepath)
 
   password = _get_key_file_encryption_password(password, prompt, filepath)
 
   # Encrypt the private key if a 'password' was passed or entered on the prompt
   if password is not None:
-    private = securesystemslib.keys.create_rsa_encrypted_pem(private, password)
+    private = keys.create_rsa_encrypted_pem(private, password)
 
   # Create intermediate directories as required
-  securesystemslib.util.ensure_parent_dir(filepath)
+  util.ensure_parent_dir(filepath)
 
   # Write PEM-encoded public key to <filepath>.pub
   file_object = tempfile.TemporaryFile()
   file_object.write(public.encode('utf-8'))
-  securesystemslib.util.persist_temp_file(file_object, filepath + '.pub')
+  util.persist_temp_file(file_object, filepath + '.pub')
 
   # Write PEM-encoded private key to <filepath>
   file_object = tempfile.TemporaryFile()
   file_object.write(private.encode('utf-8'))
-  securesystemslib.util.persist_temp_file(file_object, filepath)
+  util.persist_temp_file(file_object, filepath)
 
   return filepath
 
@@ -282,7 +283,7 @@ def generate_and_write_rsa_keypair(password, filepath=None,
     The private key filepath.
 
   """
-  securesystemslib.formats.PASSWORD_SCHEMA.check_match(password)
+  formats.PASSWORD_SCHEMA.check_match(password)
   return _generate_and_write_rsa_keypair(
       filepath=filepath, bits=bits, password=password, prompt=False)
 
@@ -385,20 +386,19 @@ def import_rsa_privatekey_from_file(filepath, password=None,
     An RSA private key object conformant with 'RSAKEY_SCHEMA'.
 
   """
-  securesystemslib.formats.PATH_SCHEMA.check_match(filepath)
-  securesystemslib.formats.RSA_SCHEME_SCHEMA.check_match(scheme)
+  formats.PATH_SCHEMA.check_match(filepath)
+  formats.RSA_SCHEME_SCHEMA.check_match(scheme)
 
   password = _get_key_file_decryption_password(password, prompt, filepath)
 
   if storage_backend is None:
-    storage_backend = securesystemslib.storage.FilesystemBackend()
+    storage_backend = FilesystemBackend()
 
   with storage_backend.get(filepath) as file_object:
     pem_key = file_object.read().decode('utf-8')
 
   # Optionally decrypt and convert PEM-encoded key to 'RSAKEY_SCHEMA' format
-  rsa_key = securesystemslib.keys.import_rsakey_from_private_pem(
-      pem_key, scheme, password)
+  rsa_key = keys.import_rsakey_from_private_pem(pem_key, scheme, password)
 
   return rsa_key
 
@@ -427,22 +427,21 @@ def import_rsa_publickey_from_file(filepath, scheme='rsassa-pss-sha256',
     An RSA public key object conformant with 'RSAKEY_SCHEMA'.
 
   """
-  securesystemslib.formats.PATH_SCHEMA.check_match(filepath)
-  securesystemslib.formats.RSA_SCHEME_SCHEMA.check_match(scheme)
+  formats.PATH_SCHEMA.check_match(filepath)
+  formats.RSA_SCHEME_SCHEMA.check_match(scheme)
 
   if storage_backend is None:
-    storage_backend = securesystemslib.storage.FilesystemBackend()
+    storage_backend = FilesystemBackend()
 
   with storage_backend.get(filepath) as file_object:
     rsa_pubkey_pem = file_object.read().decode('utf-8')
 
   # Convert PEM-encoded key to 'RSAKEY_SCHEMA' format
   try:
-    rsakey_dict = securesystemslib.keys.import_rsakey_from_public_pem(
-        rsa_pubkey_pem, scheme)
+    rsakey_dict = keys.import_rsakey_from_public_pem(rsa_pubkey_pem, scheme)
 
-  except securesystemslib.exceptions.FormatError as e:
-    raise securesystemslib.exceptions.Error('Cannot import improperly formatted'
+  except exceptions.FormatError as e:
+    raise exceptions.Error('Cannot import improperly formatted'
       ' PEM file.' + repr(str(e)))
 
   return rsakey_dict
@@ -483,41 +482,41 @@ def _generate_and_write_ed25519_keypair(filepath=None, password=None,
     The private key filepath.
 
   """
-  ed25519_key = securesystemslib.keys.generate_ed25519_key()
+  ed25519_key = keys.generate_ed25519_key()
 
   # Use passed 'filepath' or keyid as file name
   if not filepath:
     filepath = os.path.join(os.getcwd(), ed25519_key['keyid'])
 
-  securesystemslib.formats.PATH_SCHEMA.check_match(filepath)
+  formats.PATH_SCHEMA.check_match(filepath)
 
   password = _get_key_file_encryption_password(password, prompt, filepath)
 
   # Create intermediate directories as required
-  securesystemslib.util.ensure_parent_dir(filepath)
+  util.ensure_parent_dir(filepath)
 
   # Use custom JSON format for ed25519 keys on-disk
   keytype = ed25519_key['keytype']
   keyval = ed25519_key['keyval']
   scheme = ed25519_key['scheme']
-  ed25519key_metadata_format = securesystemslib.keys.format_keyval_to_metadata(
+  ed25519key_metadata_format = keys.format_keyval_to_metadata(
       keytype, scheme, keyval, private=False)
 
   # Write public key to <filepath>.pub
   file_object = tempfile.TemporaryFile()
   file_object.write(json.dumps(ed25519key_metadata_format).encode('utf-8'))
-  securesystemslib.util.persist_temp_file(file_object, filepath + '.pub')
+  util.persist_temp_file(file_object, filepath + '.pub')
 
   # Encrypt private key if we have a password, store as JSON string otherwise
   if password is not None:
-    ed25519_key = securesystemslib.keys.encrypt_key(ed25519_key, password)
+    ed25519_key = keys.encrypt_key(ed25519_key, password)
   else:
     ed25519_key = json.dumps(ed25519_key)
 
   # Write private key to <filepath>
   file_object = tempfile.TemporaryFile()
   file_object.write(ed25519_key.encode('utf-8'))
-  securesystemslib.util.persist_temp_file(file_object, filepath)
+  util.persist_temp_file(file_object, filepath)
 
   return filepath
 
@@ -550,7 +549,7 @@ def generate_and_write_ed25519_keypair(password, filepath=None):
     The private key filepath.
 
   """
-  securesystemslib.formats.PASSWORD_SCHEMA.check_match(password)
+  formats.PASSWORD_SCHEMA.check_match(password)
   return _generate_and_write_ed25519_keypair(
       filepath=filepath, password=password, prompt=False)
 
@@ -630,18 +629,17 @@ def import_ed25519_publickey_from_file(filepath):
     An ed25519 public key object conformant with 'ED25519KEY_SCHEMA'.
 
   """
-  securesystemslib.formats.PATH_SCHEMA.check_match(filepath)
+  formats.PATH_SCHEMA.check_match(filepath)
 
   # Load custom on-disk JSON formatted key and convert to its custom in-memory
   # dict key representation
-  ed25519_key_metadata = securesystemslib.util.load_json_file(filepath)
-  ed25519_key, _ = securesystemslib.keys.format_metadata_to_key(
-      ed25519_key_metadata)
+  ed25519_key_metadata = util.load_json_file(filepath)
+  ed25519_key, _ = keys.format_metadata_to_key(ed25519_key_metadata)
 
   # Check that the generic loading functions indeed loaded an ed25519 key
   if ed25519_key['keytype'] != 'ed25519':
     message = 'Invalid key type loaded: ' + repr(ed25519_key['keytype'])
-    raise securesystemslib.exceptions.FormatError(message)
+    raise exceptions.FormatError(message)
 
   return ed25519_key
 
@@ -678,18 +676,18 @@ def import_ed25519_privatekey_from_file(filepath, password=None, prompt=False,
     An ed25519 private key object conformant with 'ED25519KEY_SCHEMA'.
 
   """
-  securesystemslib.formats.PATH_SCHEMA.check_match(filepath)
+  formats.PATH_SCHEMA.check_match(filepath)
   password = _get_key_file_decryption_password(password, prompt, filepath)
 
   if storage_backend is None:
-    storage_backend = securesystemslib.storage.FilesystemBackend()
+    storage_backend = FilesystemBackend()
 
   with storage_backend.get(filepath) as file_object:
     json_str = file_object.read()
 
     # Load custom on-disk JSON formatted key and convert to its custom
     # in-memory dict key representation, decrypting it if password is not None
-    return securesystemslib.keys.import_ed25519key_from_private_json(
+    return keys.import_ed25519key_from_private_json(
         json_str, password=password)
 
 
@@ -728,41 +726,41 @@ def _generate_and_write_ecdsa_keypair(filepath=None, password=None,
     The private key filepath.
 
   """
-  ecdsa_key = securesystemslib.keys.generate_ecdsa_key()
+  ecdsa_key = keys.generate_ecdsa_key()
 
   # Use passed 'filepath' or keyid as file name
   if not filepath:
     filepath = os.path.join(os.getcwd(), ecdsa_key['keyid'])
 
-  securesystemslib.formats.PATH_SCHEMA.check_match(filepath)
+  formats.PATH_SCHEMA.check_match(filepath)
 
   password = _get_key_file_encryption_password(password, prompt, filepath)
 
   # Create intermediate directories as required
-  securesystemslib.util.ensure_parent_dir(filepath)
+  util.ensure_parent_dir(filepath)
 
   # Use custom JSON format for ecdsa keys on-disk
   keytype = ecdsa_key['keytype']
   keyval = ecdsa_key['keyval']
   scheme = ecdsa_key['scheme']
-  ecdsakey_metadata_format = securesystemslib.keys.format_keyval_to_metadata(
+  ecdsakey_metadata_format = keys.format_keyval_to_metadata(
       keytype, scheme, keyval, private=False)
 
   # Write public key to <filepath>.pub
   file_object = tempfile.TemporaryFile()
   file_object.write(json.dumps(ecdsakey_metadata_format).encode('utf-8'))
-  securesystemslib.util.persist_temp_file(file_object, filepath + '.pub')
+  util.persist_temp_file(file_object, filepath + '.pub')
 
   # Encrypt private key if we have a password, store as JSON string otherwise
   if password is not None:
-    ecdsa_key = securesystemslib.keys.encrypt_key(ecdsa_key, password)
+    ecdsa_key = keys.encrypt_key(ecdsa_key, password)
   else:
     ecdsa_key = json.dumps(ecdsa_key)
 
   # Write private key to <filepath>
   file_object = tempfile.TemporaryFile()
   file_object.write(ecdsa_key.encode('utf-8'))
-  securesystemslib.util.persist_temp_file(file_object, filepath)
+  util.persist_temp_file(file_object, filepath)
 
   return filepath
 
@@ -795,7 +793,7 @@ def generate_and_write_ecdsa_keypair(password, filepath=None):
     The private key filepath.
 
   """
-  securesystemslib.formats.PASSWORD_SCHEMA.check_match(password)
+  formats.PASSWORD_SCHEMA.check_match(password)
   return _generate_and_write_ecdsa_keypair(
       filepath=filepath, password=password, prompt=False)
 
@@ -876,13 +874,12 @@ def import_ecdsa_publickey_from_file(filepath):
     An ecdsa public key object conformant with 'ECDSAKEY_SCHEMA'.
 
   """
-  securesystemslib.formats.PATH_SCHEMA.check_match(filepath)
+  formats.PATH_SCHEMA.check_match(filepath)
 
   # Load custom on-disk JSON formatted key and convert to its custom in-memory
   # dict key representation
-  ecdsa_key_metadata = securesystemslib.util.load_json_file(filepath)
-  ecdsa_key, _ = securesystemslib.keys.format_metadata_to_key(
-      ecdsa_key_metadata)
+  ecdsa_key_metadata = util.load_json_file(filepath)
+  ecdsa_key, _ = keys.format_metadata_to_key(ecdsa_key_metadata)
 
   return ecdsa_key
 
@@ -917,21 +914,21 @@ def import_ecdsa_privatekey_from_file(filepath, password=None, prompt=False,
     An ecdsa private key object conformant with 'ED25519KEY_SCHEMA'.
 
   """
-  securesystemslib.formats.PATH_SCHEMA.check_match(filepath)
+  formats.PATH_SCHEMA.check_match(filepath)
 
   password = _get_key_file_decryption_password(password, prompt, filepath)
 
   if storage_backend is None:
-    storage_backend = securesystemslib.storage.FilesystemBackend()
+    storage_backend = FilesystemBackend()
 
   with storage_backend.get(filepath) as file_object:
     key_data = file_object.read().decode('utf-8')
 
   # Decrypt private key if we have a password, directly load JSON otherwise
   if password is not None:
-    key_object = securesystemslib.keys.decrypt_key(key_data, password)
+    key_object = keys.decrypt_key(key_data, password)
   else:
-    key_object = securesystemslib.util.load_json_string(key_data)
+    key_object = util.load_json_string(key_data)
 
   # Raise an exception if an unexpected key type is imported.
   # NOTE: we support keytype's of ecdsa-sha2-nistp256 and ecdsa-sha2-nistp384
@@ -941,12 +938,11 @@ def import_ecdsa_privatekey_from_file(filepath, password=None, prompt=False,
   if key_object['keytype'] not in['ecdsa', 'ecdsa-sha2-nistp256',
       'ecdsa-sha2-nistp384']:
     message = 'Invalid key type loaded: ' + repr(key_object['keytype'])
-    raise securesystemslib.exceptions.FormatError(message)
+    raise exceptions.FormatError(message)
 
   # Add "keyid_hash_algorithms" so that equal ecdsa keys with different keyids
   # can be associated using supported keyid_hash_algorithms.
-  key_object['keyid_hash_algorithms'] = \
-      securesystemslib.settings.HASH_ALGORITHMS
+  key_object['keyid_hash_algorithms'] = settings.HASH_ALGORITHMS
 
   return key_object
 
@@ -983,7 +979,7 @@ def import_publickeys_from_file(filepaths, key_types=None):
     key_types = [KEY_TYPE_RSA] * len(filepaths)
 
   if len(key_types) != len(filepaths):
-    raise securesystemslib.exceptions.FormatError(
+    raise exceptions.FormatError(
         "Pass equal amount of 'filepaths' (got {}) and 'key_types (got {}), "
         "or no 'key_types' at all to default to '{}'.".format(
         len(filepaths), len(key_types), KEY_TYPE_RSA))
@@ -1000,7 +996,7 @@ def import_publickeys_from_file(filepaths, key_types=None):
       key = import_ecdsa_publickey_from_file(filepath)
 
     else:
-      raise securesystemslib.exceptions.FormatError(
+      raise exceptions.FormatError(
           "Unsupported key type '{}'. Must be '{}', '{}' or '{}'.".format(
           key_types[idx], KEY_TYPE_RSA, KEY_TYPE_ED25519, KEY_TYPE_ECDSA))
 
@@ -1059,7 +1055,7 @@ def import_privatekey_from_file(filepath, key_type=None, password=None,
         filepath, password=password, prompt=prompt)
 
   else:
-    raise securesystemslib.exceptions.FormatError(
+    raise exceptions.FormatError(
         "Unsupported key type '{}'. Must be '{}', '{}' or '{}'.".format(
         key_type, KEY_TYPE_RSA, KEY_TYPE_ED25519, KEY_TYPE_ECDSA))
 
